from django.contrib import admin
from .models import Food, Meal, Portion, Preferences


class PortionAdmin(admin.ModelAdmin):
    list_display = ("date", "food", "quantity", "meal", "user")
    list_filter = ["date"]


admin.site.register(Food)
admin.site.register(Meal)
admin.site.register(Portion, PortionAdmin)
admin.site.register(Preferences)



from django.http import HttpResponse
from django.utils import timezone
from django import forms
from . import api, models
from django.views import generic
from django.template import loader
from django.shortcuts import render, redirect, get_object_or_404
from django.contrib.auth.decorators import login_required
from django.contrib.auth.mixins import LoginRequiredMixin
from django.urls import reverse


class DaysView(LoginRequiredMixin, generic.ListView):
    template_name = "django_nutrition/days.html"
    context_object_name = "days"

    def get_queryset(self):
        start_date = timezone.now() - timezone.timedelta(weeks=4)
        _days = api.DayTotal.split_days(
            models.Portion.objects.filter(date__gte=start_date, user=self.request.user)
        )
        return sorted(_days, key=lambda d: d.date, reverse=True)


@login_required
def day(request, day_str):
    meals = api.MealTotal.split_portions(models.Portion.objects.filter(date=day_str))
    template = loader.get_template("django_nutrition/day.html")
    context = {
        "meals": meals,
        "date": day_str,
        "calories": sum(m.calories for m in meals),
    }
    return HttpResponse(template.render(context, request))


class PortionForm(forms.ModelForm):
    class Meta:
        model = models.Portion
        fields = ["date", "quantity", "food", "meal", "note"]
        widgets = {
            "date": forms.DateInput(
                attrs={
                    # DaisyUI input styling
                    "class": "input input-bordered w-full max-w-xs",
                    "type": "date",
                }
            ),
            "quantity": forms.NumberInput(
                attrs={
                    # DaisyUI input styling
                    "class": "input input-bordered w-full max-w-xs",
                    "step": 0.01,
                }
            ),
            "food": forms.Select(
                attrs={
                    # DaisyUI select style
                    "class": "select select-bordered w-full max-w-xs",
                }
            ),
            "meal": forms.Select(
                attrs={
                    # DaisyUI select style
                    "class": "select select-bordered w-full max-w-xs",
                }
            ),
            "note": forms.TextInput(
                attrs={
                    # DaisyUI input styling
                    "class": "input input-bordered w-full max-w-xs",
                }
            ),
        }

    def __init__(self, *args, **kwargs):
        user = kwargs.pop("user", None)
        assert user  # sanity (should always be logged in here & user provided)
        super(PortionForm, self).__init__(*args, **kwargs)
        self.fields["food"].queryset = models.Food.objects.filter(user=user)
        self.fields["meal"].queryset = models.Meal.objects.filter(user=user)


@login_required
def add_or_edit_portion(request, pk=None):
    _portion = get_object_or_404(models.Portion, pk=pk) if pk else None

    if request.method == "POST":
        form = PortionForm(request.POST, instance=_portion, user=request.user)
        if form.is_valid():
            portion_instance = form.save(commit=False)
            portion_instance.user = request.user
            portion_instance.save()

            selected_date = form.cleaned_data["date"]
            day_str = selected_date.strftime("%Y-%m-%d")
            return redirect("day", day_str=day_str)
    else:
        default_date = request.GET.get("date", timezone.now().date())
        form = PortionForm(
            initial={"date": default_date}, instance=_portion, user=request.user
        )

    return render(request, "django_nutrition/add_portion.html", {"form": form})


@login_required
def delete_portion(request, pk):
    _record = get_object_or_404(models.Portion, pk=pk)
    if request.method == "POST":
        portion_date = _record.date
        _record.delete()
        return redirect(reverse("day", args=[portion_date]))

    return render(
        request, "django_nutrition/portion_confirm_delete.html", {"portion": _record}
    )


class UserPreferencesForm(forms.ModelForm):
    class Meta:
        model = models.Preferences
        fields = ["max_calories", "theme"]
        # widgets = {
        #     'max_calories': forms.NumberInput(attrs={
        #           # DaisyUI input styling
        #         'class': 'input input-bordered w-full max-w-xs',
        #         'step': 0.01}),
        #     'theme': forms.Select(attrs={
        #         # DaisyUI select style
        #         'class': 'select select-bordered w-full max-w-xs',
        #     }),
        #   }

    def __init__(self, *args, **kwargs):
        _prefs = kwargs.get("instance")
        assert isinstance(_prefs, models.Preferences)  # sanity
        super(UserPreferencesForm, self).__init__(*args, **kwargs)
        self.fields["max_calories"].initial = _prefs.max_calories


@login_required
def user_preferences(request):
    preferences, created = models.Preferences.objects.get_or_create(user=request.user)

    if request.method == "POST":
        form = UserPreferencesForm(request.POST, instance=preferences)
        if form.is_valid():
            form.save()
            return redirect("days")
    else:
        form = UserPreferencesForm(instance=preferences)

    return render(request, "django_nutrition/user-preferences.html", {"form": form})


def style_test(request):
    return render(request, "django_nutrition/style-test.html")


class FoodsView(LoginRequiredMixin, generic.ListView):
    template_name = "django_nutrition/foods.html"
    context_object_name = "foods"

    def get_queryset(self):
        return models.Food.objects.filter(user=self.request.user).order_by("name")


class FoodForm(forms.ModelForm):
    class Meta:
        model = models.Food
        fields = ["name", "calories"]


@login_required
def add_or_edit_food(request, pk=None):
    _food = get_object_or_404(models.Food, pk=pk) if pk else None
    if request.method == "POST":
        form = FoodForm(request.POST, instance=_food)
        if form.is_valid():
            _food = form.save(commit=False)
            _food.user = request.user
            _food.save()
            return redirect("foods")
    else:
        form = FoodForm(instance=_food)
    return render(
        request, "django_nutrition/food_form.html", {"form": form, "food": _food}
    )


# @login_required
# def delete_food(request, pk):
#     item = get_object_or_404(models.Food, pk=pk)
#     if request.method == 'POST':
#         item.delete()
#         return redirect('foods')
#
#     return render(request, 'django_nutrition/food_confirm_delete.html', {'item': item})



from django.urls import path
from django.contrib.auth import views as auth_views

from . import views, api

urlpatterns = [
    path("", views.DaysView.as_view(), name="days"),
    path("foods", views.FoodsView.as_view(), name="foods"),
    path("day/<str:day_str>", views.day, name="day"),
    path("user-preferences/", views.user_preferences, name="user_preferences"),
    path("api/events/", api.days, name="day-events"),
    path(
        "login/",
        auth_views.LoginView.as_view(template_name="django_nutrition/login.html"),
        name="nutrition_login",
    ),
    path("style-test/", views.style_test, name="style_test"),
    path("portion/new", views.add_or_edit_portion, name="add_or_edit_portion"),
    path(
        "portion/edit/<int:pk>/", views.add_or_edit_portion, name="add_or_edit_portion"
    ),
    path("portion/delete/<int:pk>/", views.delete_portion, name="delete_portion"),
    path("food/new", views.add_or_edit_food, name="add_or_edit_food"),
    path("food/edit/<int:pk>/", views.add_or_edit_food, name="add_or_edit_food"),
    # path('food/delete/<int:pk>/', views.delete_food, name='delete_food'),
]



from django.test import TestCase
from django.utils import timezone
from django.contrib.auth.models import User
from django.urls import reverse

from .models import Portion, Food
from .api import MealTotal


class MealTotalTests(TestCase):
    def test_split_portions(self):
        user = User.objects.create_user("test2", "1234")

        food_100 = Food.objects.create(name="food 100", calories=100, user=user)
        Portion.objects.create(
            food=food_100, quantity=1.0, date=timezone.now(), user=user
        )

        food_200 = Food.objects.create(name="food 200", calories=200, user=user)
        Portion.objects.create(
            food=food_200, quantity=1.0, date=timezone.now(), user=user
        )

        food_300 = Food.objects.create(name="food 300", calories=300, user=user)
        Portion.objects.create(
            food=food_300, quantity=1.0, date=timezone.now(), user=user
        )

        portions = Portion.objects.all()

        meals = MealTotal.split_portions(portions)
        self.assertEqual(len(meals), 1)
        self.assertEqual(meals[0].calories, 600)


class ViewsTest(TestCase):
    USERNAME = "testuser"
    PASSWORD = "testpassword"

    def setUp(self):
        _user = User.objects.create_user(
            username=ViewsTest.USERNAME, password=ViewsTest.PASSWORD
        )

        foods = [
            Food.objects.create(name="food 100", calories=100, user=_user),
            Food.objects.create(name="food 200", calories=200, user=_user),
            Food.objects.create(name="food 300", calories=300, user=_user),
            Food.objects.create(name="food 1000", calories=1000, user=_user),
        ]

        _today = [
            Portion.objects.create(
                food=_f, quantity=1.0, date=timezone.now(), user=_user
            )
            for _f in foods
        ]

        _yesterday = [
            Portion.objects.create(
                food=_f,
                quantity=1.0,
                date=timezone.now() - timezone.timedelta(days=1),
                user=_user,
            )
            for _f in foods
        ]

        _tomorrow = [
            Portion.objects.create(
                food=_f,
                quantity=1.0,
                date=timezone.now() + timezone.timedelta(days=1),
                user=_user,
            )
            for _f in foods
        ]

    def test_add_to_cart(self):
        response = self.client.get(reverse("days"))
        # expect to be redirected to login
        self.assertEqual(response.status_code, 302)

        self.client.login(username=ViewsTest.USERNAME, password=ViewsTest.PASSWORD)
        response = self.client.get(reverse("days"))
        self.assertEqual(response.status_code, 200)
        # dumb ... should do a better test than this (e.g. dates)
        self.assertContains(response, "Calories")

    def test_days_list_api(self):
        response = self.client.get(reverse("day-events"))
        self.assertEqual(response.status_code, 403)

        self.client.login(username=ViewsTest.USERNAME, password=ViewsTest.PASSWORD)
        response = self.client.get(reverse("day-events"))
        self.assertEqual(response.status_code, 200)
        days_list = response.json()
        # very stupid hard-coded based on test setup data
        assert len(days_list) == 2



from django.db import models
from django.utils import timezone
from django.contrib.auth.models import User
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from django.http import HttpRequest


class Food(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    name = models.CharField(max_length=200)
    calories = models.FloatField()

    def __str__(self):
        return self.name


class Meal(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    name = models.CharField(max_length=200)

    def __str__(self):
        return self.name


class Portion(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    date = models.DateField(default=timezone.now)
    quantity = models.FloatField(default=1)
    note = models.CharField(max_length=200, blank=True)
    food = models.ForeignKey(Food, on_delete=models.CASCADE)
    meal = models.ForeignKey(Meal, on_delete=models.SET_NULL, null=True, blank=True)

    def calories(self):
        return self.food.calories * self.quantity

    def __str__(self):
        return f"{self.food.name} ({self.quantity})"


class Preferences(models.Model):
    DEFAULT_MAX_CALORIES = 2000
    DEFAULT_THEME = "light"

    AVAILABLE_THEMES = [
        "light",
        "dark",
        "cupcake",
        "bumblebee",
        "emerald",
        "corporate",
        "synthwave",
        "retro",
        "cyberpunk",
        "valentine",
        "halloween",
        "garden",
        "forest",
        "aqua",
        "lofi",
        "pastel",
        "fantasy",
        "wireframe",
        "black",
        "luxury",
        "dracula",
        "cmyk",
        "autumn",
        "business",
        "acid",
        "lemonade",
        "night",
        "coffee",
        "winter",
        "dim",
        "nord",
        "sunset",
    ]

    user = models.OneToOneField(User, on_delete=models.CASCADE)
    max_calories = models.FloatField(blank=True, null=True)
    theme = models.CharField(
        max_length=200,
        choices=[(t, t) for t in AVAILABLE_THEMES],
        blank=True,
        null=True,
    )

    def __str__(self):
        return f'{self.user}, max calories: {self.max_calories}, theme: "{self.theme}"'

    @staticmethod
    def current_preferences(request: "HttpRequest") -> dict:
        return_value = {
            "max_calories": Preferences.DEFAULT_MAX_CALORIES,
            "theme": Preferences.DEFAULT_THEME,
        }
        if request.user.is_authenticated:
            try:
                _prefs = Preferences.objects.get(user=request.user)
                return_value["max_calories"] = (
                    _prefs.max_calories or Preferences.DEFAULT_MAX_CALORIES
                )
                return_value["theme"] = _prefs.theme or Preferences.DEFAULT_THEME
            except Preferences.DoesNotExist:
                pass  # ok (prefs are optional)

        return return_value



from datetime import datetime
from typing import Iterable, List, TYPE_CHECKING
from django.utils import timezone
from . import models
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework import serializers

if TYPE_CHECKING:
    from django.http import HttpRequest


class MealTotal:
    def __init__(self, name: str, portions: Iterable[models.Portion]):
        self.name = name
        self.portions = portions
        self.calories = sum(p.calories() for p in portions)

    def to_dict(self):
        return {"name": self.name, "calories": self.calories, "portions": self.portions}

    @staticmethod
    def split_portions(portions: Iterable[models.Portion]) -> List["MealTotal"]:
        _meals = dict()
        for p in portions:
            meal_name = p.meal.name if p.meal else "other"
            _m = _meals.setdefault(meal_name, []).append(p)
        return [MealTotal(_name, _portions) for _name, _portions in _meals.items()]


class DayTotal:
    def __init__(self, date: timezone, portions: Iterable[models.Portion]):
        self.date = date
        self.calories = sum(p.calories() for p in portions)
        self.meals = MealTotal.split_portions(portions)

    @staticmethod
    def split_days(portions: Iterable[models.Portion]) -> List["DayTotal"]:
        portions_per_day = dict()
        for p in portions:
            portions_per_day.setdefault(p.date, []).append(p)

        return [DayTotal(date, portions) for date, portions in portions_per_day.items()]


class FullDayEvent:
    def __init__(self, day: DayTotal, over: False):
        self.title = str(int(day.calories + 0.5))
        self.start = day.date
        self.end = day.date
        self.allDay = True
        self.description = f"{day.calories} calories"
        # self.display = 'background'
        self.display = "auto"
        self.backgroundColor = "red" if over else "green"
        self.textColor = "white"
        self.url = f"/nutrition/day/{day.date}"


class FullDayEventSerializer(serializers.Serializer):
    title = serializers.CharField(max_length=200)
    start = serializers.DateField()
    end = serializers.DateField()
    description = serializers.CharField(max_length=200)
    allDay = serializers.BooleanField()
    display = serializers.CharField(max_length=200)
    backgroundColor = serializers.CharField(max_length=200)
    textColor = serializers.CharField(max_length=200)
    url = serializers.CharField(max_length=200)


@api_view(["GET"])
@permission_classes([IsAuthenticated])
def days(request: "HttpRequest"):
    start = request.query_params.get("start", None)
    if start:
        # FullCalendar doesn't send timezone info
        start_date = datetime.fromisoformat(start)
    else:
        start_date = timezone.now() - timezone.timedelta(weeks=4)

    end = request.query_params.get("end", None)
    if end:
        # FullCalendar doesn't send timezone info
        end_date = datetime.fromisoformat(end)
    else:
        end_date = timezone.now()

    _days = DayTotal.split_days(
        models.Portion.objects.filter(
            date__gte=start_date, date__lte=end_date, user=request.user
        )
    )

    _prefs = models.Preferences.current_preferences(request)

    def _make_event(day: DayTotal):
        return FullDayEvent(day, over=day.calories > _prefs["max_calories"])

    serializer = FullDayEventSerializer(map(_make_event, _days), many=True)
    return Response(serializer.data)
